/*---------------------- C++ source file -----------------------
 Project: Game

     Game

 Developer: Pavel Bagretsov
 Website: http://gamedesign.narod.ru/
 EMail: gamedesign@narod.ru
--------------------------------------------------------------*/

#pragma comment (lib,"winmm.lib")

#include "tEngine\tEngine.h"

tEngine* engine = new tEngine;

struct VERTEX {
    float x,y,z; // 
    DWORD color; // 
};

//  
VERTEX vertices[] = {
	{-1.0f,1.0f,0.0f,0x00ff0000},
	{-1.0f,-1.0f,0.0f,0x0000ff00},
	{1.0f,-1.0f,0.0f,0x000000ff},
	{1.0f,-1.0f,0.0f,0x000000ff},
	{1.0f,1.0f,0.0f,0x0000ff00},
	{-1.0f,1.0f,0.0f,0x00ff0000},
	{-1.0f,1.0f,1.0f,0x00ff0000},
	{-1.0f,-1.0f,1.0f,0x0000ff00},
	{1.0f,-1.0f,1.0f,0x000000ff},
	{1.0f,-1.0f,1.0f,0x000000ff},
	{1.0f,1.0f,1.0f,0x0000ff00},
	{-1.0f,1.0f,1.0f,0x00ff0000},
};

int vb1;
int effect1;

/*--------------------------------------------------------------
  
--------------------------------------------------------------*/
void Null(void)
{
}

/*--------------------------------------------------------------
    
--------------------------------------------------------------*/
void SetMatrices(void)
{
    D3DXMATRIX d3dx_matrix_normal;
    D3DXMATRIX d3dx_matrix_world;
    D3DXMATRIX d3dx_matrix_view;
    D3DXMATRIX d3dx_matrix_proj;
    D3DXMATRIX d3dx_matrix1,d3dx_matrix2;

    //   
    D3DXMatrixRotationX(&d3dx_matrix1,timeGetTime()/1000.0f);
    D3DXMatrixRotationY(&d3dx_matrix2,timeGetTime()/500.0f);
    D3DXMatrixMultiply(&d3dx_matrix_normal,&d3dx_matrix1,&d3dx_matrix2);

    //   
    D3DXMatrixRotationX(&d3dx_matrix1,timeGetTime()/1000.0f);
    D3DXMatrixRotationY(&d3dx_matrix2,timeGetTime()/500.0f);
    D3DXMatrixMultiply(&d3dx_matrix_world,&d3dx_matrix1,&d3dx_matrix2);

    //   
    D3DXMatrixLookAtLH(&d3dx_matrix_view,&D3DXVECTOR3(0.0f,0.0f,-5.0f),
                       &D3DXVECTOR3(0.0f,0.0f,0.0f),
                       &D3DXVECTOR3(0.0f,1.0f,0.0f));

    //   
    D3DXMatrixPerspectiveFovLH(&d3dx_matrix_proj,D3DX_PI/4,
                               (float)engine->GetRenderWidth()/(float)engine->GetRenderHeight(),
                               1.0f,1000.0f);

    D3DXMatrixTranspose(&d3dx_matrix1,&d3dx_matrix_normal);
    D3DXMatrixIdentity(&d3dx_matrix2);
    D3DXMatrixMultiply(&d3dx_matrix2,&d3dx_matrix2,&d3dx_matrix_world);
    D3DXMatrixMultiply(&d3dx_matrix2,&d3dx_matrix2,&d3dx_matrix_view);
    D3DXMatrixMultiply(&d3dx_matrix2,&d3dx_matrix2,&d3dx_matrix_proj);
    D3DXMatrixTranspose(&d3dx_matrix2,&d3dx_matrix2);

    //     
    engine->SetVertexShaderConstant(0,&d3dx_matrix1,4);
    engine->SetVertexShaderConstant(4,&d3dx_matrix2,4);
}

/*--------------------------------------------------------------
 
--------------------------------------------------------------*/
void Render(void)
{
    if(engine->BeginRender())
    {
        SetMatrices();

        engine->BeginEffect(effect1);

        engine->PassEffect(effect1,0);
        engine->SetVertexBuffer(vb1,0);
        engine->DrawRender(4);

        engine->EndEffect(effect1);
    
        engine->EndRender();
    }

    if(GetAsyncKeyState(VK_F1))
        engine->InitRender(0,0,800,600,32);
    if(GetAsyncKeyState(VK_F2))
        engine->InitRender(100,100,400,300);
}

/*--------------------------------------------------------------
   
--------------------------------------------------------------*/
void Main(void)
{
    try
    {
        engine->InitRender(100,100,400,300);

        vb1 = engine->LoadVertexBuffer(&vertices,sizeof(vertices),TVERTEXXYZ|TVERTEXDIFFUSE);
        effect1 = engine->LoadEffect("Base\\Effects\\first.eff");

        engine->SetEngineThread(Render);
    }
    catch(tError &error)
    {
        error.AddErrorHistory(" <-thread 0x%x",&Main);
        throw;
    }
}

/*--------------------------------------------------------------
   Game.exe
--------------------------------------------------------------*/
int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    try
    {
        //  
        engine->InitEngine();
        //    
        engine->SetEngineThread(Main);
        //   
        engine->RunEngine();
        //  
        engine->ReleaseEngine();
    }
    catch(tError &error)
    {
        error.AddErrorHistory(" <-WinMain <-Game.exe");

        const char* errorMessage = error.GetErrorMessage();

        //      log-
        engine->LogMessage("\n%s\n",errorMessage);

        //      
        MessageBox(NULL,errorMessage,"Critical error!",MB_OK|MB_ICONERROR);

        delete engine;
        return 1;
    }

    delete engine;
    return 0;
}
