/*---------------------- C++ source file -----------------------
 Project: tEngine

  D3D

 Developer: Pavel Bagretsov
 Website: http://gamedesign.narod.ru/
 EMail: gamedesign@narod.ru
--------------------------------------------------------------*/

#include "tD3D.h"

/*--------------------------------------------------------------
 tD3D::tD3D()

   
--------------------------------------------------------------*/
tD3D::tD3D()
{
    m_d3d = NULL;
    m_d3dDevice = NULL;

    ZeroMemory(&m_d3dPP,sizeof(m_d3dPP));
    ZeroMemory(&m_d3dCaps,sizeof(m_d3dCaps));
}

/*--------------------------------------------------------------
 tD3D::~tD3D()

 
--------------------------------------------------------------*/
tD3D::~tD3D()
{
    if(m_d3d)
        m_d3d->Release();
    if(m_d3dDevice)
        m_d3dDevice->Release();
}

/*--------------------------------------------------------------
 tD3D::InitD3D()

  D3D
--------------------------------------------------------------*/
void tD3D::InitD3D(int x,int y,int width,int height,int color)
{
    HRESULT hResult;
    D3DADAPTER_IDENTIFIER8 adapterIdentifier;

    try
    {
        //  
        tWindow::InitWindow(x,y,width,height);
    }
    catch(tError &error)
    {
        error.AddErrorHistory(" <-tD3D::InitD3D");
        throw;
    }

    tLogFile::LogMessage("Create D3D ... ");

    //  Direct3D
    m_d3d = Direct3DCreate8(D3D_SDK_VERSION);
    if(!m_d3d)
        throw tError(__FILE__,__LINE__,"tD3D::InitD3D",
                     "Can't create D3D!");

    tLogFile::LogMessage("Ok.\n");

    //    -
    hResult = m_d3d->GetAdapterIdentifier(D3DADAPTER_DEFAULT,0,&adapterIdentifier);
    if(hResult!=D3D_OK)
        throw tError(__FILE__,__LINE__,"tD3D::InitD3D",
                     "Fail to get adapter information! (%s)",DXGetErrorString8(hResult));

    //    -  log-
    tLogFile::LogMessage("  Description: %s\n",adapterIdentifier.Description);
    tLogFile::LogMessage("  Driver: %s\n",adapterIdentifier.Driver);
    tLogFile::LogMessage("  Driver Version: %d.%d.%d.%d\n",
                         HIWORD(adapterIdentifier.DriverVersion.HighPart),
                         LOWORD(adapterIdentifier.DriverVersion.HighPart),
                         HIWORD(adapterIdentifier.DriverVersion.LowPart),
                         LOWORD(adapterIdentifier.DriverVersion.LowPart));

    //    -  log-
    tLogFile::LogMessage("  Available video modes:\n");
    for(int i=0;i<tD3D::GetD3DDisplayModeCount();i++)
    {
        tD3D::GetD3DDisplayMode(i,&m_d3dDM);
        tLogFile::LogMessage("    %dx%dx%d %d\n",
                             m_d3dDM.Width,
                             m_d3dDM.Height,
                             tD3D::ConvertD3DFormat(m_d3dDM.Format),
                             m_d3dDM.RefreshRate);
    }

    tLogFile::LogMessage("Create D3D Device ... ");

    //    
    m_d3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT,&m_d3dDM);

    //   Direct3D
    if(!color)
    {
        //    D3D
        m_d3dPP.BackBufferWidth = width;
        m_d3dPP.BackBufferHeight = height;
        m_d3dPP.BackBufferFormat = m_d3dDM.Format;
        m_d3dPP.BackBufferCount = 1;

        m_d3dPP.MultiSampleType = D3DMULTISAMPLE_NONE;

        m_d3dPP.SwapEffect = D3DSWAPEFFECT_DISCARD;
        m_d3dPP.hDeviceWindow = m_hWindow;
        m_d3dPP.Windowed = TRUE;

        m_d3dPP.EnableAutoDepthStencil = TRUE;
        m_d3dPP.AutoDepthStencilFormat = D3DFMT_D16;
    }
    else
    {
        //    D3D
        m_d3dPP.BackBufferWidth = width;
        m_d3dPP.BackBufferHeight = height;
        if(color==32)
            m_d3dPP.BackBufferFormat = D3DFMT_X8R8G8B8;
        else
            m_d3dPP.BackBufferFormat = D3DFMT_R5G6B5;
        m_d3dPP.BackBufferCount = 1;

        m_d3dPP.MultiSampleType = D3DMULTISAMPLE_NONE;

        m_d3dPP.SwapEffect = D3DSWAPEFFECT_DISCARD;
        m_d3dPP.hDeviceWindow = m_hWindow;
        m_d3dPP.Windowed = FALSE;

        m_d3dPP.EnableAutoDepthStencil = TRUE;
        m_d3dPP.AutoDepthStencilFormat = D3DFMT_D16;
    }

    //   Direct3D    
    hResult = m_d3d->CreateDevice(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,m_hWindow,
                                  D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                  &m_d3dPP,&m_d3dDevice);
    if(hResult!=D3D_OK)
        throw tError(__FILE__,__LINE__,"tD3D::InitD3D",
                     "Can't create D3D Device! / x - %d, y - %d, width - %d, height - %d, color - %d (%s)",
                     x,y,width,height,color,DXGetErrorString8(hResult));

    tLogFile::LogMessage("Ok.\n");

    //     Direct3D
    hResult = m_d3dDevice->GetDeviceCaps(&m_d3dCaps);
    if(hResult!=D3D_OK)
        throw tError(__FILE__,__LINE__,"tD3D::InitD3D",
                     "Fail to get D3D Device information! (%s)",DXGetErrorString8(hResult));
}

/*--------------------------------------------------------------
 tD3D::ReleaseD3D()

  D3D
--------------------------------------------------------------*/
void tD3D::ReleaseD3D(void)
{
    if(m_d3d)
    {
        m_d3d->Release();
        m_d3d = NULL;
    }

    if(m_d3dDevice)
    {
        m_d3dDevice->Release();
        m_d3dDevice = NULL;
    }

    ZeroMemory(&m_d3dPP,sizeof(m_d3dPP));
    ZeroMemory(&m_d3dCaps,sizeof(m_d3dCaps));
}

/*--------------------------------------------------------------
 tD3D::GetD3DDisplayModeCount()

    -
--------------------------------------------------------------*/
int tD3D::GetD3DDisplayModeCount(void)
{
    if(m_d3d)
        return (int)m_d3d->GetAdapterModeCount(D3DADAPTER_DEFAULT);
    return 0;
}

/*--------------------------------------------------------------
 tD3D::GetD3DDisplayMode()

    -
--------------------------------------------------------------*/
void tD3D::GetD3DDisplayMode(int mode,D3DDISPLAYMODE* displayMode)
{
    HRESULT hResult;

    if(m_d3d)
        hResult = m_d3d->EnumAdapterModes(D3DADAPTER_DEFAULT,mode,displayMode);

    if(hResult!=D3D_OK||!m_d3d)
        ZeroMemory(displayMode,sizeof(*displayMode));
}

/*--------------------------------------------------------------
 tD3D::ConvertD3DFormat()

  D3DFORMAT   
   
--------------------------------------------------------------*/
int tD3D::ConvertD3DFormat(D3DFORMAT format)
{
    switch(format)
    {
        case D3DFMT_R8G8B8:
            return 24;
        case D3DFMT_A8R8G8B8:
            return 32;
        case D3DFMT_X8R8G8B8:
            return 32;
        case D3DFMT_R5G6B5:
            return 16;
        case D3DFMT_X1R5G5B5:
            return 16;
        case D3DFMT_A1R5G5B5:
            return 16;
        case D3DFMT_A4R4G4B4:
            return 16;
        case D3DFMT_R3G3B2:
            return 8;
        case D3DFMT_A8R3G3B2:
            return 16;
        case D3DFMT_X4R4G4B4:
            return 16;
    }
    return 0;
}