/*---------------------- C++ source file -----------------------
 Project: tEngine

   

 Developer: Pavel Bagretsov
 Website: http://gamedesign.narod.ru/
 EMail: gamedesign@narod.ru
--------------------------------------------------------------*/

#include "tEffects.h"

/*--------------------------------------------------------------
 tEffects::tEffects()

   
--------------------------------------------------------------*/
tEffects::tEffects()
{
    //  
    for(int i=0;i<TEFFECTSMAXCOUNT;i++)
        m_vector.push_back(tEFFECT());

    //  
    for(i=TEFFECTSMAXCOUNT;i>0;i--)
        m_stack.push(i);
}

/*--------------------------------------------------------------
 tEffects::~tEffects()

 
--------------------------------------------------------------*/
tEffects::~tEffects()
{
    for(int i=0;i<TEFFECTSMAXCOUNT;i++)
    {
        if(m_vector[i].technique)
            m_vector[i].technique->Release();

        if(m_vector[i].effect)
            m_vector[i].effect->Release();

        m_vector[i].fileName.erase();
    }

    m_vector.clear();
}

/*--------------------------------------------------------------
 tEffects::LoadEffect()

    
--------------------------------------------------------------*/
int tEffects::LoadEffect(const char* name)
{
    HRESULT hResult;
    LPD3DXBUFFER buffer;
    D3DXEFFECT_DESC desc;

    //     
    int id = m_stack.top();
    if(m_stack.size()!=1)
        m_stack.pop();

    //   
    hResult = D3DXCompileEffectFromFile(name,&buffer,NULL);
    if(hResult!=D3D_OK)
        throw tError(__FILE__,__LINE__,"tEffects::LoadEffect",
                     "Fail to compile effect file! / name - %s (%s)",name,DXGetErrorString8(hResult));

    //  
    hResult = D3DXCreateEffect(m_d3dDevice,
                               buffer->GetBufferPointer(),buffer->GetBufferSize(),
                               0,
                               &m_vector[id-1].effect);
    if(hResult!=D3D_OK)
        throw tError(__FILE__,__LINE__,"tEffects::LoadEffect",
                     "Fail to create effect! / name - %s (%s)",name,DXGetErrorString8(hResult));

    buffer->Release();

    //   
    m_vector[id-1].fileName = name;

    //   
    m_vector[id-1].effect->GetDesc(&desc);

    //  
    for(int i=0;i<(int)desc.Techniques;i++)
    {
        m_vector[id-1].effect->GetTechnique(i,&m_vector[id-1].technique);
        if(m_vector[id-1].technique->Validate()==D3D_OK)
            break;
        else
            m_vector[id-1].technique = NULL;
    }

    return id;
}

/*--------------------------------------------------------------
 tEffects::ReleaseEffect()

  
--------------------------------------------------------------*/
void tEffects::ReleaseEffect(int id)
{
    if(m_vector[id-1].effect)
    {
        if(m_vector[id-1].technique)
        {
            m_vector[id-1].technique->Release();
            m_vector[id-1].technique = NULL;
        }

        m_vector[id-1].effect->Release();
        m_vector[id-1].effect = NULL;

        m_vector[id-1].fileName.erase();

        //    
        m_stack.push(id);
    }
}

/*--------------------------------------------------------------
 tEffects::BeginEffect()

  
--------------------------------------------------------------*/
inline int tEffects::BeginEffect(int id)
{
    int passes = 0;

    if(m_vector[id-1].technique)
        m_vector[id-1].technique->Begin((unsigned int*)&passes);

    return passes;
}

/*--------------------------------------------------------------
 tEffects::EndEffect()

  
--------------------------------------------------------------*/
inline void tEffects::EndEffect(int id)
{
    if(m_vector[id-1].technique)
        m_vector[id-1].technique->End();
}

/*--------------------------------------------------------------
 tEffects::PassEffect()

  
--------------------------------------------------------------*/
inline void tEffects::PassEffect(int id,int pass)
{
    if(m_vector[id-1].technique)
        m_vector[id-1].technique->Pass(pass);
}

/*--------------------------------------------------------------
 tEffects::Reset()

  
--------------------------------------------------------------*/
void tEffects::Reset(void)
{
    for(int i=0;i<TEFFECTSMAXCOUNT;i++)
        if(!m_vector[i].fileName.empty())
        {
            LPD3DXBUFFER buffer;

            //   
            D3DXCompileEffectFromFile(m_vector[i].fileName.c_str(),&buffer,NULL);

            //  
            D3DXCreateEffect(m_d3dDevice,
                             buffer->GetBufferPointer(),buffer->GetBufferSize(),
                             0,
                             &m_vector[i].effect);

            buffer->Release();

            D3DXEFFECT_DESC desc;

            //   
            m_vector[i].effect->GetDesc(&desc);

            //  
            for(int j=0;j<(int)desc.Techniques;i++)
            {
                m_vector[i].effect->GetTechnique(j,&m_vector[i].technique);
                if(m_vector[i].technique->Validate()==D3D_OK)
                    break;
                else
                    m_vector[i].technique = NULL;
            }
        }
}
