/*---------------------- C++ source file -----------------------
 Project: tEngine

    

 Developer: Pavel Bagretsov
 Website: http://gamedesign.narod.ru/
 EMail: gamedesign@narod.ru
--------------------------------------------------------------*/

#include "tIndexBuffers.h"

/*--------------------------------------------------------------
 tIndexBuffers::tIndexBuffers()

   
--------------------------------------------------------------*/
tIndexBuffers::tIndexBuffers()
{
    //  
    for(int i=0;i<TINDEXBUFFERSMAXCOUNT;i++)
        m_vector.push_back(tINDEXBUFFER());

    //  
    for(i=TINDEXBUFFERSMAXCOUNT;i>0;i--)
        m_stack.push(i);
}

/*--------------------------------------------------------------
 tIndexBuffers::~tIndexBuffers()

 
--------------------------------------------------------------*/
tIndexBuffers::~tIndexBuffers()
{
    for(int i=0;i<TINDEXBUFFERSMAXCOUNT;i++)
        if(m_vector[i].buffer)
            m_vector[i].buffer->Release();

    m_vector.clear();
}

/*--------------------------------------------------------------
 tIndexBuffers::LoadIndexBuffer()

   
--------------------------------------------------------------*/
int tIndexBuffers::LoadIndexBuffer(void* source,int size,int indexFormat)
{
    HRESULT hResult;
    void* dest;

    //     
    int id = m_stack.top();
    if(m_stack.size()!=1)
        m_stack.pop();

    //   
    if(m_d3dDevice)
        hResult = m_d3dDevice->CreateIndexBuffer(size,
                                                 0,
                                                 (D3DFORMAT)indexFormat,
                                                 D3DPOOL_MANAGED,
                                                 &m_vector[id-1].buffer);
    if(hResult!=D3D_OK||!m_d3dDevice)
        throw tError(__FILE__,__LINE__,"tIndexBuffers::LoadIndexBuffer",
                     "Fail to create index buffer! (%s)",DXGetErrorString8(hResult));

    m_vector[id-1].indexFormat = indexFormat;

    //     
    m_vector[id-1].buffer->Lock(0,size,(BYTE**)&dest,0);
    memcpy(dest,source,size);
    m_vector[id-1].buffer->Unlock();

    return id;
}

/*--------------------------------------------------------------
 tIndexBuffers::ReleaseIndexBuffer()

   
--------------------------------------------------------------*/
void tIndexBuffers::ReleaseIndexBuffer(int id)
{
    if(m_vector[id-1].buffer)
    {
        m_vector[id-1].buffer->Release();

        m_vector[id-1].buffer = NULL;
        m_vector[id-1].indexFormat = 0;

        //    
        m_stack.push(id);
    }
}

/*--------------------------------------------------------------
 tIndexBuffers::SetIndexBuffer()

   
--------------------------------------------------------------*/
inline void tIndexBuffers::SetIndexBuffer(int id,int start)
{
    if(m_vector[id-1].buffer)
        m_d3dDevice->SetIndices(m_vector[id-1].buffer,start);
}
