/*---------------------- C++ source file -----------------------
 Project: tEngine

  

 Developer: Pavel Bagretsov
 Website: http://gamedesign.narod.ru/
 EMail: gamedesign@narod.ru
--------------------------------------------------------------*/

#include "tRender.h"

/*--------------------------------------------------------------
 tRender::tRender()

   
--------------------------------------------------------------*/
tRender::tRender()
{
}

/*--------------------------------------------------------------
 tRender::~tRender()

 
--------------------------------------------------------------*/
tRender::~tRender()
{
}

/*--------------------------------------------------------------
 tRender::InitRender()

  
--------------------------------------------------------------*/
void tRender::InitRender(int x,int y,int width,int height,int color)
{
    try
    {
        if(!m_d3dDevice)
            tD3D::InitD3D(x,y,width,height,color);
        else
        {
            if(!color)
            {
                //     D3D
                m_d3dPP.BackBufferWidth = width;
                m_d3dPP.BackBufferHeight = height;
                m_d3dPP.BackBufferFormat = m_d3dDM.Format;
                m_d3dPP.Windowed = TRUE;
    
                tRender::Reset();

                //     
                SetWindowPos(m_hWindow,NULL,x,y,width,height,0);
                //  
                CloseWindow(m_hWindow);
                //  
                ShowWindow(m_hWindow,SW_SHOWNORMAL);
            }
            else
            {
                //     D3D
                m_d3dPP.BackBufferWidth = width;
                m_d3dPP.BackBufferHeight = height;
                if(color==32)
                    m_d3dPP.BackBufferFormat = D3DFMT_X8R8G8B8;
                else
                    m_d3dPP.BackBufferFormat = D3DFMT_R5G6B5;
                m_d3dPP.Windowed = FALSE;

                //     
                SetWindowPos(m_hWindow,NULL,x,y,width,height,0);
                //  
                CloseWindow(m_hWindow);
                //  
                ShowWindow(m_hWindow,SW_SHOWNORMAL);

                tRender::Reset();
            }

            TWINDOWSIZE = false;
        }
    }
    catch(tError &error)
    {
        error.AddErrorHistory(" <-tRender::InitRender");
    }
}

/*--------------------------------------------------------------
 tRender::ReleaseRender()

  
--------------------------------------------------------------*/
void tRender::ReleaseRender(void)
{
    for(int i=1;i<=TTEXTURESMAXCOUNT;i++)
        tTextures::ReleaseTexture(i);

    for(i=1;i<=TVERTEXBUFFERSMAXCOUNT;i++)
        tVertexBuffers::ReleaseVertexBuffer(i);

    for(i=1;i<=TINDEXBUFFERSMAXCOUNT;i++)
        tIndexBuffers::ReleaseIndexBuffer(i);

    for(i=1;i<=TEFFECTSMAXCOUNT;i++)
        tEffects::ReleaseEffect(i);

    tD3D::ReleaseD3D();
}

/*--------------------------------------------------------------
 tRender::BeginRender()

   
--------------------------------------------------------------*/
inline bool tRender::BeginRender(void)
{
    HRESULT hResult;
    
    //     
    if(!TWINDOWACTIVE)
        return false;

    //    Direct3D
    hResult = m_d3dDevice->TestCooperativeLevel();
    
    //  
    if(hResult==D3DERR_DEVICELOST)
        return false;
    
    //   
    if(hResult==D3DERR_DEVICENOTRESET)
    {
        tRender::Reset();
        return false;
    }

    //     (   )
    if(tRender::IsRenderSize()&&m_d3dPP.Windowed==TRUE)
    {
        m_d3dPP.BackBufferWidth = tRender::GetRenderWidth();
        m_d3dPP.BackBufferHeight = tRender::GetRenderHeight();

        tRender::Reset();

        TWINDOWSIZE = false;

        return false;
    }

    //   
    m_d3dDevice->Clear(0,NULL,D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,D3DCOLOR_XRGB(0,0,0),1.0f,0);
    //   
    m_d3dDevice->BeginScene();

    return true;
}

/*--------------------------------------------------------------
 tRender::EndRender()

   
--------------------------------------------------------------*/
inline void tRender::EndRender(void)
{
    //   
    m_d3dDevice->EndScene();
    m_d3dDevice->Present(NULL,NULL,NULL,NULL);
}

/*--------------------------------------------------------------
 tRender::DrawRender()

  
--------------------------------------------------------------*/
inline void tRender::DrawRender(int count)
{
    m_d3dDevice->DrawPrimitive(D3DPT_TRIANGLELIST,0,count);
}

/*--------------------------------------------------------------
 tRender::SetVertexShaderConstant()

     
--------------------------------------------------------------*/
inline void tRender::SetVertexShaderConstant(int constantRegister,void* constantData,int constantCount)
{
    m_d3dDevice->SetVertexShaderConstant(constantRegister,constantData,constantCount);
}

/*--------------------------------------------------------------
 tRender::SetPixelShaderConstant()

     
--------------------------------------------------------------*/
inline void tRender::SetPixelShaderConstant(int constantRegister,void* constantData,int constantCount)
{
    m_d3dDevice->SetPixelShaderConstant(constantRegister,constantData,constantCount);
}

/*--------------------------------------------------------------
 tRender::GetRenderWidth()

   
--------------------------------------------------------------*/
inline int tRender::GetRenderWidth(void)
{
    RECT rect;

    GetClientRect(m_hWindow,&rect);
    
    return rect.right-rect.left;
}
    
/*--------------------------------------------------------------
 tRender::GetRenderHeight()

   
--------------------------------------------------------------*/
inline int tRender::GetRenderHeight(void)
{
    RECT rect;

    GetClientRect(m_hWindow,&rect);
    
    return rect.bottom-rect.top;
}

/*--------------------------------------------------------------
 tRender::Reset()

  
--------------------------------------------------------------*/
void tRender::Reset(void)
{
    //   
    for(int i=0;i<TEFFECTSMAXCOUNT;i++)
        if(tEffects::m_vector[i].effect)
        {
            if(tEffects::m_vector[i].technique)
                tEffects::m_vector[i].technique->Release();

            tEffects::m_vector[i].effect->Release();
        }

    //   Direct3D
    m_d3dDevice->Reset(&m_d3dPP);

    //  
    tEffects::Reset();
}
