/*---------------------- C++ source file -----------------------
 Project: tEngine

   

 Developer: Pavel Bagretsov
 Website: http://gamedesign.narod.ru/
 EMail: gamedesign@narod.ru
--------------------------------------------------------------*/

#include "tTextures.h"

/*--------------------------------------------------------------
 tTextures::tTextures()

   
--------------------------------------------------------------*/
tTextures::tTextures()
{
    //  
    for(int i=0;i<TTEXTURESMAXCOUNT;i++)
        m_vector.push_back(tTEXTURE());

    //  
    for(i=TTEXTURESMAXCOUNT;i>0;i--)
        m_stack.push(i);
}

/*--------------------------------------------------------------
 tTextures::~tTextures()

 
--------------------------------------------------------------*/
tTextures::~tTextures()
{
    for(int i=0;i<TTEXTURESMAXCOUNT;i++)
        if(m_vector[i].texture)
            m_vector[i].texture->Release();

    m_vector.clear();
}

/*--------------------------------------------------------------
 tTextures::LoadTexture()

    
--------------------------------------------------------------*/
int tTextures::LoadTexture(const char* name)
{
    HRESULT hResult;

    //     
    int id = m_stack.top();
    if(m_stack.size()!=1)
        m_stack.pop();

    //  
    hResult = D3DXCreateTextureFromFileEx(m_d3dDevice,
                                          name,
                                          D3DX_DEFAULT,D3DX_DEFAULT,D3DX_DEFAULT,
                                          0,
                                          D3DFMT_UNKNOWN,
                                          D3DPOOL_MANAGED,
                                          D3DX_DEFAULT,
                                          D3DX_DEFAULT,
                                          0,
                                          NULL,
                                          NULL,
                                          &m_vector[id-1].texture);
    if(hResult!=D3D_OK)
        throw tError(__FILE__,__LINE__,"tTextures::LoadTexture",
                     "Fail to load texture! / name - %s (%s)",name,DXGetErrorString8(hResult));
    return id;
}

/*--------------------------------------------------------------
 tTextures::ReleaseTexture()

  
--------------------------------------------------------------*/
void tTextures::ReleaseTexture(int id)
{
    if(m_vector[id-1].texture)
    {
        m_vector[id-1].texture->Release();
        m_vector[id-1].texture = NULL;

        //    
        m_stack.push(id);
    }
}

/*--------------------------------------------------------------
 tTextures::SetTexture()

  
--------------------------------------------------------------*/
inline void tTextures::SetTexture(int id,int stage)
{
    if(m_vector[id-1].texture)
        m_d3dDevice->SetTexture(stage,m_vector[id-1].texture);
}
