/*---------------------- C++ source file -----------------------
 Project: tEngine

    

 Developer: Pavel Bagretsov
 Website: http://gamedesign.narod.ru/
 EMail: gamedesign@narod.ru
--------------------------------------------------------------*/

#include "tVertexBuffers.h"

/*--------------------------------------------------------------
 tVertexBuffers::tVertexBuffers()

   
--------------------------------------------------------------*/
tVertexBuffers::tVertexBuffers()
{
    //  
    for(int i=0;i<TVERTEXBUFFERSMAXCOUNT;i++)
        m_vector.push_back(tVERTEXBUFFER());

    //  
    for(i=TVERTEXBUFFERSMAXCOUNT;i>0;i--)
        m_stack.push(i);
}

/*--------------------------------------------------------------
 tVertexBuffers::~tVertexBuffers()

 
--------------------------------------------------------------*/
tVertexBuffers::~tVertexBuffers()
{
    for(int i=0;i<TVERTEXBUFFERSMAXCOUNT;i++)
        if(m_vector[i].buffer)
            m_vector[i].buffer->Release();

    m_vector.clear();
}

/*--------------------------------------------------------------
 tVertexBuffers::LoadVertexBuffer()

   
--------------------------------------------------------------*/
int tVertexBuffers::LoadVertexBuffer(void* source,int size,int vertexFormat)
{
    HRESULT hResult;
    void* dest;

    //     
    int id = m_stack.top();
    if(m_stack.size()!=1)
        m_stack.pop();

    //   
    if(m_d3dDevice)
        hResult = m_d3dDevice->CreateVertexBuffer(size,
                                                  0,
                                                  vertexFormat,
                                                  D3DPOOL_MANAGED,
                                                  &m_vector[id-1].buffer);
    if(hResult!=D3D_OK||!m_d3dDevice)
        throw tError(__FILE__,__LINE__,"tVertexBuffers::LoadVertexBuffer",
                     "Fail to create vertex buffer! (%s)",DXGetErrorString8(hResult));

    m_vector[id-1].vertexFormat = vertexFormat;

    //     
    m_vector[id-1].buffer->Lock(0,size,(BYTE**)&dest,0);
    memcpy(dest,source,size);
    m_vector[id-1].buffer->Unlock();

    return id;
}

/*--------------------------------------------------------------
 tVertexBuffers::ReleaseVertexBuffer()

   
--------------------------------------------------------------*/
void tVertexBuffers::ReleaseVertexBuffer(int id)
{
    if(m_vector[id-1].buffer)
    {
        m_vector[id-1].buffer->Release();

        m_vector[id-1].buffer = NULL;
        m_vector[id-1].vertexFormat = 0;

        //    
        m_stack.push(id);
    }
}

/*--------------------------------------------------------------
 tVertexBuffers::SetVertexBuffer()

     
--------------------------------------------------------------*/
inline void tVertexBuffers::SetVertexBuffer(int id,int stream)
{
    if(m_vector[id-1].buffer)
        m_d3dDevice->SetStreamSource(stream,m_vector[id-1].buffer,D3DXGetFVFVertexSize(m_vector[id-1].vertexFormat));
}
