/*---------------------- C++ source file -----------------------
 Project: tEngine

  

 Developer: Pavel Bagretsov
 Website: http://gamedesign.narod.ru/
 EMail: gamedesign@narod.ru
--------------------------------------------------------------*/

#include "tWindow.h"

/*--------------------------------------------------------------
 
--------------------------------------------------------------*/
const char* TWINDOWCLASSNAME = "tEngine Window Class";
const char* TWINDOWNAME	= "http://gamedesign.narod.ru/ | Open project - Game";

bool TWINDOWACTIVE = true;
bool TWINDOWSIZE = true;

/*--------------------------------------------------------------
 WindowMsgProc() 

    
--------------------------------------------------------------*/
LRESULT WINAPI WindowMsgProc(HWND hWnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
    switch(msg)
    {
        case WM_ACTIVATE:
            if(HIWORD(wParam))
                TWINDOWACTIVE = false;
            else
                TWINDOWACTIVE = true;
            break;
        case WM_SIZE:
            if(TWINDOWSIZE)
                TWINDOWSIZE = false;
            else
                TWINDOWSIZE = true;
            break;
        case WM_DESTROY:
            PostQuitMessage(0);
            break;
    }
    return DefWindowProc(hWnd,msg,wParam,lParam);
}

/*--------------------------------------------------------------
 tWindow::tWindow()

   
--------------------------------------------------------------*/
tWindow::tWindow()
{
    m_hWindow = NULL;
}

/*--------------------------------------------------------------
 tWindow::~tWindow()

 
--------------------------------------------------------------*/
tWindow::~tWindow()
{
}

/*--------------------------------------------------------------
 tWindow::InitWindow()

  
--------------------------------------------------------------*/
void tWindow::InitWindow(int x,int y,int width,int height)
{
    WNDCLASSEX windowClass;

    windowClass.cbSize = sizeof(WNDCLASSEX);
    windowClass.style = CS_OWNDC|CS_HREDRAW|CS_VREDRAW;
    windowClass.lpfnWndProc = WindowMsgProc;
    windowClass.cbClsExtra = 0;
    windowClass.cbWndExtra = 0;
    windowClass.hInstance = GetModuleHandle(NULL);
    windowClass.hIcon = LoadIcon(NULL,IDI_APPLICATION);
    windowClass.hCursor = LoadCursor(NULL,IDC_ARROW);
    windowClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
    windowClass.lpszMenuName = NULL;
    windowClass.lpszClassName = TWINDOWCLASSNAME;
    windowClass.hIconSm = NULL;

    tLogFile::LogMessage("Register window class - \"%s\" ... ",TWINDOWCLASSNAME);

    //   
    if(!GetClassInfoEx(GetModuleHandle(NULL),TWINDOWCLASSNAME,&windowClass))
        if(!RegisterClassEx(&windowClass))
            throw tError(__FILE__,__LINE__,"tWindow::InitWindow",
                         "Can't register window class! / class name - %s",TWINDOWCLASSNAME);

    tLogFile::LogMessage("Ok.\n");

    tLogFile::LogMessage("Create window - \"%s\" ... ",TWINDOWNAME);

    //  
    m_hWindow = CreateWindowEx(0,TWINDOWCLASSNAME,TWINDOWNAME,
                               WS_OVERLAPPEDWINDOW,
                               x,y,width,height,
                               NULL,
                               NULL,
                               GetModuleHandle(NULL),
                               NULL);
    if(m_hWindow)
        ShowWindow(m_hWindow,SW_SHOWNORMAL);
    else
        throw tError(__FILE__,__LINE__,"tWindow::InitWindow",
                     "Can't create window! / name - %s",TWINDOWNAME);

    tLogFile::LogMessage("Ok.\n");
}

/*--------------------------------------------------------------
 tWindow::ReleaseWindow()

  
--------------------------------------------------------------*/
void tWindow::ReleaseWindow(void)
{
    DestroyWindow(m_hWindow);
}
